@@ -28,21 +28,21 @@ from jinja2 import StrictUndefined
 
 from airflow.exceptions import TaskDeferred
 from airflow.models import DAG, DagRun, TaskInstance
-from airflow.models.serialized_dag import SerializedDagModel
 from airflow.providers.amazon.aws.operators.emr import EmrCreateJobFlowOperator
 from airflow.providers.amazon.aws.triggers.emr import EmrCreateJobFlowTrigger
 from airflow.providers.amazon.aws.utils.waiter import WAITER_POLICY_NAME_MAPPING, WaitPolicy
+from airflow.utils.state import DagRunState
+from airflow.utils.types import DagRunType
+
+from tests_common.test_utils.dag import sync_dag_to_db
+from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS
+from unit.amazon.aws.utils.test_template_fields import validate_template_fields
+from unit.amazon.aws.utils.test_waiter import assert_expected_waiter_type
 
 try:
     from airflow.sdk import timezone
 except ImportError:
     from airflow.utils import timezone  # type: ignore[attr-defined,no-redef]
-from airflow.utils.state import DagRunState
-from airflow.utils.types import DagRunType
-
-from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS
-from unit.amazon.aws.utils.test_template_fields import validate_template_fields
-from unit.amazon.aws.utils.test_waiter import assert_expected_waiter_type
 
 TASK_ID = "test_task"
 
@@ -102,13 +102,12 @@ class TestEmrCreateJobFlowOperator:
         assert self.operator.region_name == "ap-southeast-2"
 
     @pytest.mark.db_test
-    def test_render_template(self, session, clean_dags_and_dagruns):
+    def test_render_template(self, session, clean_dags_dagruns_and_dagbundles, testing_dag_bundle):
         self.operator.job_flow_overrides = self._config
         if AIRFLOW_V_3_0_PLUS:
             from airflow.models.dag_version import DagVersion
 
-            self.operator.dag.sync_to_db()
-            SerializedDagModel.write_dag(self.operator.dag, bundle_name="testing")
+            sync_dag_to_db(self.operator.dag)
             dag_version = DagVersion.get_latest_version(self.operator.dag.dag_id)
             ti = TaskInstance(task=self.operator, dag_version_id=dag_version.id)
             dag_run = DagRun(
@@ -154,15 +153,16 @@ class TestEmrCreateJobFlowOperator:
         assert self.operator.job_flow_overrides == expected_args
 
     @pytest.mark.db_test
-    def test_render_template_from_file(self, mocked_hook_client, session, clean_dags_and_dagruns):
+    def test_render_template_from_file(
+        self, mocked_hook_client, session, clean_dags_dagruns_and_dagbundles, testing_dag_bundle
+    ):
         self.operator.job_flow_overrides = "job.j2.json"
         self.operator.params = {"releaseLabel": "5.11.0"}
 
         if AIRFLOW_V_3_0_PLUS:
             from airflow.models.dag_version import DagVersion
 
-            self.operator.dag.sync_to_db()
-            SerializedDagModel.write_dag(self.operator.dag, bundle_name="testing")
+            sync_dag_to_db(self.operator.dag)
             dag_version = DagVersion.get_latest_version(self.operator.dag.dag_id)
             ti = TaskInstance(task=self.operator, dag_version_id=dag_version.id)
             dag_run = DagRun(
@@ -238,11 +238,12 @@ class TestEmrCreateJobFlowOperator:
     def test_create_job_flow_deferrable(self, mocked_hook_client):
         """
         Test to make sure that the operator raises a TaskDeferred exception
-        if run in deferrable mode.
+        if run in deferrable mode and wait_policy is set.
         """
         mocked_hook_client.run_job_flow.return_value = RUN_JOB_FLOW_SUCCESS_RETURN
 
         self.operator.deferrable = True
+        self.operator.wait_policy = WaitPolicy.WAIT_FOR_COMPLETION
         with pytest.raises(TaskDeferred) as exc:
             self.operator.execute(self.mock_context)
 
@@ -250,5 +251,17 @@ class TestEmrCreateJobFlowOperator:
             "Trigger is not a EmrCreateJobFlowTrigger"
         )
 
+    def test_create_job_flow_deferrable_no_wait(self, mocked_hook_client):
+        """
+        Test to make sure that the operator does NOT raise a TaskDeferred exception
+        if run in deferrable mode but wait_policy is not set.
+        """
+        mocked_hook_client.run_job_flow.return_value = RUN_JOB_FLOW_SUCCESS_RETURN
+
+        self.operator.deferrable = True
+        # wait_policy is None by default
+        result = self.operator.execute(self.mock_context)
+        assert result == JOB_FLOW_ID
+
     def test_template_fields(self):
         validate_template_fields(self.operator)
